require 'rb-readline'

module TodoList
  class CommandLineInterface

    def run
      command = ARGV[0]
      command ||= "print_list"

      if respond_to?(command)
        execute(command)
      else
        print_command_error(%(Unknown command #{bold(command)}.))
      end
    end

    def add
      description = ARGV[1..-1].join(" ")

      @task_list << Task.new(description) unless description.empty?

      print_list
    end

    def edit
      task = get_todo_tasks[get_task_index]

      if ARGV.size > 2
        task.description = ARGV[2..-1].join(" ")
      else
        RbReadline.prefill_prompt(task.description)
        task.description = Readline.readline("Edit description: ")
      end

      print_list
    end

    def done
      get_todo_tasks[get_task_index].done = true

      print_list
    end

    def remove
      @task_list.delete(get_todo_tasks[get_task_index])

      print_list
    end

    def clean
      @task_list = TaskList.new(get_todo_tasks)

      print_list
    end

    def wipe
      @task_list = TaskList.new

      print_list
    end

    def print_list
      if @task_list.empty?
        puts %(Nothing to do !\n\nType "help" command for more information.)
        return
      end

      unless get_todo_tasks.empty?
        puts "[TODO]"
        get_todo_tasks.each_with_index do |e, i|
          puts "#{i + 1} - #{e.description.capitalize}"
        end
        puts unless get_done_tasks.empty?
      end

      unless get_done_tasks.empty?
        puts "[DONE]"
        get_done_tasks.each do |e|
          puts "- #{e.description.capitalize}"
        end
      end
    end

    def print_help
      puts <<-ENDTEXT
Available commands :
    help
    add    #{underline("task_description")}
    edit   #{underline("task_number")} [#{underline("new_description")}]
    done   #{underline("task_number")}
    remove #{underline("task_number")}
    clean
    wipe
ENDTEXT
    end

    alias_method :help, :print_help

    private

    def execute(command)
      @task_list = TaskList.load(SAVE_PATH)

      send(command)

      @task_list.save(SAVE_PATH)
    end

    def get_done_tasks
      @task_list.select { |e| e.done? }
    end

    def get_todo_tasks
      @task_list.select { |e| !e.done? }
    end

    def print_command_error(error_message)
      print("#{error_message}\n\n")

      print_help
    end

    def underline(string)
      "\033[4m#{string}\033[0m"
    end

    def bold(string)
      "\033[1m#{string}\033[0m"
    end

    def get_task_index
      task_index = ARGV[1].to_i - 1

      unless task_index.between?(0, get_todo_tasks.size - 1)
        puts "Please type a valid task number."
        exit
      end

      task_index
    end
  end
end

module RbReadline
  def self.prefill_prompt(str)
    @rl_prefill = str
    @rl_startup_hook = :rl_prefill_hook
  end

  def self.rl_prefill_hook
    rl_insert_text @rl_prefill if @rl_prefill
    @rl_startup_hook = nil
  end
end